/* 
   orxonox - the future of 3D-vertical-scrollers

   Copyright (C) 2004 orx

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   ### File Specific:
   main-programmer: Benjamin Grauer
   co-programmer: ...
*/

#include "object.h"

Object::Object ()
{

  initialize();

  importFile ("reaphigh.obj");

  finalize();
}

Object::Object(char* fileName)
{
  initialize();

  importFile (fileName);

  finalize();
}

Object::Object(char* fileName, float scaling)
{
  initialize();
  scaleFactor = scaling;

  importFile (fileName);

  finalize();
}

bool Object::initialize (void)
{
  if (verbose >=3)
    printf("new 3D-Object is being created\n"); 
  faceMode = -1;
  if ( (listNumber = glGenLists(1)) == 0 )
    {
      printf ("list could not be created for this Object\n");
      return false;
    }

  mtlFileName = "";
  scaleFactor = 1;
  vertices = new Array();
  normals = new Array();
  vTexture = new Array();

  glNewList (listNumber, GL_COMPILE);
  glEnableClientState (GL_VERTEX_ARRAY);
  glEnableClientState (GL_NORMAL_ARRAY);
  //  glEnableClientState (GL_TEXTURE_COORD_ARRAY);

  return true;
}

bool Object::importFile (char* fileName)
{
  if (verbose >=3)
    printf("preparing to read in file: %s\n", fileName);   
  objFileName = fileName;
  this->readFromObjFile (objFileName);
  return true;
}

bool Object::finalize(void)
{
  if (verbose >=3)
    printf("finalizing the 3D-Object\n"); 
  OBJ_FILE->close();
  glEnd();
  glEndList();
  return true;
}

void Object::draw (void)
{
  if (verbose >=3)
    printf("drawing the 3D-Object\n"); 
  glCallList (listNumber);
}


bool Object::readFromObjFile (char* fileName)
{
  OBJ_FILE = new ifstream(fileName);
  if (!OBJ_FILE->is_open())
    {
      if (verbose >=1)
	printf ("unable to open .OBJ file: %s\n Loading Box Object instead.\n", fileName);
      BoxObject();
      return false;
    }
  objFileName = fileName;
  char Buffer[500];
  while(!OBJ_FILE->eof())
    {
      OBJ_FILE->getline(Buffer, 500);
      if (verbose >=4)
	printf ("Read input line: %s\n",Buffer);
      

      // case vertice
      if (!strncmp(Buffer, "v ", 2))
	{
	  readVertex(Buffer+2);
	}

      // case face
      else if (!strncmp(Buffer, "f ", 2))
	{
	  readFace (Buffer+2);
	}
      
      else if (!strncmp(Buffer, "mtllib", 6))
	{
	  readMtlLib (Buffer+7);
	}

      else if (!strncmp(Buffer, "usemtl", 6))
	{
	  readUseMtl (Buffer+7);
	}

      // case VertexNormal
      else if (!strncmp(Buffer, "vn ", 2))
      {
	readVertexNormal(Buffer+3);
      }

      // case vt
      else if (!strncmp(Buffer, "vt ", 2))
      {
	readVertexTexture(Buffer+3);
      }
	  

    }
  

  
  
}


bool Object::readVertex (char* vertexString)
{
  readVertices = true;
  char subbuffer1[20];
  char subbuffer2[20];
  char subbuffer3[20];
  sscanf (vertexString, "%s %s %s", subbuffer1, subbuffer2, subbuffer3);
  if (verbose >= 3)
    printf ("reading in a vertex: %s %s %s\n", subbuffer1, subbuffer2, subbuffer3);
  vertices->addEntry(atof(subbuffer1)*scaleFactor, atof(subbuffer2)*scaleFactor, atof(subbuffer3)*scaleFactor);
  return true;
}

bool Object::readFace (char* faceString)
{
  if (readVertices == true)
    {
      vertices->finalizeArray();
      glVertexPointer(3, GL_FLOAT, 0, vertices->getArray());
      normals->finalizeArray();
      glNormalPointer(GL_FLOAT, 0, normals->getArray());
      vTexture->finalizeArray();
    }

  readVertices = false;
  char subbuffer1[20];
  char subbuffer2[20];
  char subbuffer3[20];
  char subbuffer4[20] ="";
  sscanf (faceString, "%s %s %s %s", subbuffer1, subbuffer2, subbuffer3, subbuffer4);
  if (!strcmp(subbuffer4, ""))
    {
      if (faceMode != 3)
	{
	  if (faceMode != -1)
	    glEnd();
	  glBegin(GL_TRIANGLES);
	}
      
      faceMode = 3;
      if (verbose >=3)
	printf ("found triag: %s, %s, %s\n", subbuffer1, subbuffer2, subbuffer3);
      addGLElement(subbuffer1);
      addGLElement(subbuffer2);
      addGLElement(subbuffer3);
      return true;
    }
  else
    {
      if (faceMode != 4)
	{
	  if (faceMode != -1)
	    glEnd();
	  glBegin(GL_QUADS);
	}
      faceMode = 4;
      if (verbose >=3 )
	printf ("found quad: %s, %s, %s, %s\n", subbuffer1, subbuffer2, subbuffer3, subbuffer4);
      addGLElement(subbuffer1);
      addGLElement(subbuffer2);
      addGLElement(subbuffer3);
      addGLElement(subbuffer4);
      return true;
    }
}

bool Object::addGLElement (char* elementString)
{
  if (verbose >=3)
    printf ("importing grafical Element.... including to openGL\n");
  char* vertex = elementString;

  char* texture;
  texture = strstr (vertex, "/");
  texture[0] = '\0';
  texture ++;
  glTexCoord2fv(vTexture->getArray()+(atoi(texture)-1)*2);

  char* normal;
  if ((normal = strstr (texture, "/")) !=NULL)
    {
      normal[0] = '\0';
      normal ++;
      //glArrayElement(atoi(vertex)-1);
      glNormal3fv(normals->getArray() +(atoi(normal)-1)*3);
    }
  glVertex3fv(vertices->getArray() +(atoi(vertex)-1)*3);

}

bool Object::readVertexNormal (char* normalString)
{
  readVertices = true;
  char subbuffer1[20];
  char subbuffer2[20];
  char subbuffer3[20];
  sscanf (normalString, "%s %s %s", subbuffer1, subbuffer2, subbuffer3);
  if (verbose >=3 )
    printf("found vertex-Normal %s, %s, %s\n", subbuffer1,subbuffer2,subbuffer3);
  normals->addEntry(atof(subbuffer1), atof(subbuffer2), atof(subbuffer3));
  return true;
}

bool Object::readVertexTexture (char* vTextureString)
{
  readVertices = true;
  char subbuffer1[20];
  char subbuffer2[20];
  sscanf (vTextureString, "%s %s", subbuffer1, subbuffer2);
  if (verbose >=3 )
    printf("found vertex-Texture %s, %s\n", subbuffer1,subbuffer2);
  vTexture->addEntry(atof(subbuffer1));
  vTexture->addEntry(atof(subbuffer2));
  return true;
}


bool Object::readMtlLib (char* mtlFile)
{
  MTL_FILE = new ifstream (mtlFile);
  if (!MTL_FILE->is_open())
    {
      if (verbose >= 1)
	printf ("unable to open file: %s\n", mtlFile);
      return false;
    }
  mtlFileName = mtlFile;
  if (verbose >=2)
    printf ("Opening mtlFile: %s\n", mtlFileName);
  char Buffer[500];
  vertices = new Array();
  material = new Material();
  Material* tmpMat = material;
  while(!MTL_FILE->eof())
    {
      MTL_FILE->getline(Buffer, 500);
      if (verbose >= 4)
	printf("found line in mtlFile: %s\n", Buffer);
      

      // create new Material
      if (!strncmp(Buffer, "newmtl ", 2))
	{
	  tmpMat = tmpMat->addMaterial(Buffer+7);
	  //	  printf ("%s, %p\n", tmpMat->getName(), tmpMat);
	}
      // setting a illumMode
      else if (!strncmp(Buffer, "illum", 5))
	{
	  tmpMat->setIllum(Buffer+6);

	}
      // setting Diffuse Color
      else if (!strncmp(Buffer, "Kd", 2))
	{
	  tmpMat->setDiffuse(Buffer+3);
	}
      // setting Ambient Color
      else if (!strncmp(Buffer, "Ka", 2))
	{
	  tmpMat->setAmbient(Buffer+3);
	}
      // setting Specular Color
      else if (!strncmp(Buffer, "Ks", 2))
	{
	  tmpMat->setSpecular(Buffer+3);
	}
    }
  return true;
}

bool Object::readUseMtl (char* matString)
{
  if (!strcmp (mtlFileName, ""))
    {
      if (verbose >= 1)
	printf ("Not using new defined material, because no mtlFile found yet\n");
      return false;
    }
      
  if (faceMode != -1)
    glEnd();
  faceMode = 0;
  if (verbose >= 2)
    printf ("using material %s for coming Faces.\n", matString);
  material->search(matString)->select();
}


void Object::BoxObject(void)
{
  readVertex ("-0.500000 -0.500000 0.500000");
  readVertex ("0.500000 -0.500000 0.500000");
  readVertex ("-0.500000 0.500000 0.500000");
  readVertex ("0.500000 0.500000 0.500000");
  readVertex ("-0.500000 0.500000 -0.500000");
  readVertex ("0.500000 0.500000 -0.500000");
  readVertex ("-0.500000 -0.500000 -0.500000");
  readVertex ("0.500000 -0.500000 -0.500000");
  readVertexTexture ("0.000000 0.000000");
  readVertexTexture ("1.000000 0.000000");
  readVertexTexture ("0.000000 1.000000");
  readVertexTexture ("1.000000 1.000000");
  readVertexTexture ("0.000000 2.000000");
  readVertexTexture ("1.000000 2.000000");
  readVertexTexture ("0.000000 3.000000");
  readVertexTexture ("1.000000 3.000000");
  readVertexTexture ("0.000000 4.000000");
  readVertexTexture ("1.000000 4.000000");
  readVertexTexture ("2.000000 0.000000");
  readVertexTexture ("2.000000 1.000000");
  readVertexTexture ("-1.000000 0.000000");
  readVertexTexture ("-1.000000 1.000000");
  
  readVertexNormal ("0.000000 0.000000 1.000000");
  readVertexNormal ("0.000000 0.000000 1.000000");
  readVertexNormal ("0.000000 0.000000 1.000000");
  readVertexNormal ("0.000000 0.000000 1.000000");
  readVertexNormal ("0.000000 1.000000 0.000000");
  readVertexNormal ("0.000000 1.000000 0.000000");
  readVertexNormal ("0.000000 1.000000 0.000000");
  readVertexNormal ("0.000000 1.000000 0.000000");
  readVertexNormal ("0.000000 0.000000 -1.000000");
  readVertexNormal ("0.000000 0.000000 -1.000000");
  readVertexNormal ("0.000000 0.000000 -1.000000");
  readVertexNormal ("0.000000 0.000000 -1.000000");
  readVertexNormal ("0.000000 -1.000000 0.000000");
  readVertexNormal ("0.000000 -1.000000 0.000000");
  readVertexNormal ("0.000000 -1.000000 0.000000");
  readVertexNormal ("0.000000 -1.000000 0.000000");
  readVertexNormal ("1.000000 0.000000 0.000000");
  readVertexNormal ("1.000000 0.000000 0.000000");
  readVertexNormal ("1.000000 0.000000 0.000000");
  readVertexNormal ("1.000000 0.000000 0.000000");
  readVertexNormal ("-1.000000 0.000000 0.000000");
  readVertexNormal ("-1.000000 0.000000 0.000000");
  readVertexNormal ("-1.000000 0.000000 0.000000");
  readVertexNormal ("-1.000000 0.000000 0.000000");

  readFace ("1/1/1 2/2/2 4/4/3 3/3/4");
  readFace ("3/3/5 4/4/6 6/6/7 5/5/8");
  readFace ("5/5/9 6/6/10 8/8/11 7/7/12");
  readFace ("7/7/13 8/8/14 2/10/15 1/9/16");
  readFace ("2/2/17 8/11/18 6/12/19 4/4/20");
  readFace ("7/13/21 1/1/22 3/3/23 5/14/24");
}
